from __future__ import absolute_import, division, print_function

from flask import Flask, request
import requests

from  PIL import Image

import os
import argparse
import cv2
import numpy as np
import sys
import glob
import importlib.util

# import time

# This code was run on Raspberry Pi 4


def tf(image_name, directory, counter, camera):
    
    # @@@@@@@@@@ Code by EdjeElectronics 
    # @@@@@@@@@@ From https://github.com/EdjeElectronics/TensorFlow-Lite-Object-Detection-on-Android-and-Raspberry-Pi/blob/master/Raspberry_Pi_Guide.md 
    # Follow instruction on github link for how to set up and run


    # @@@@@@@@@@ Object Detection Model From https://www.tensorflow.org/lite/models/object_detection/overview

    # Define and parse input arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('--modeldir', help='Folder the .tflite file is located in',
                        )
    parser.add_argument('--graph', help='Name of the .tflite file, if different than detect.tflite',
                        default='detect.tflite')
    parser.add_argument('--labels', help='Name of the labelmap file, if different than labelmap.txt',
                        default='labelmap.txt')
    parser.add_argument('--threshold', help='Minimum confidence threshold for displaying detected objects',
                        default=0.5)
    parser.add_argument('--image', help='Name of the single image to perform detection on. To run detection on multiple images, use --imagedir',
                        default=None)
    parser.add_argument('--imagedir', help='Name of the folder containing images to perform detection on. Folder must contain only images.',
                        default=None)
    parser.add_argument('--edgetpu', help='Use Coral Edge TPU Accelerator to speed up detection',
                        action='store_true')
    

    args = parser.parse_args()

    #MODEL_NAME = args.modeldir
    GRAPH_NAME = args.graph
    LABELMAP_NAME = args.labels
    min_conf_threshold = float(args.threshold)
    use_TPU = args.edgetpu

    MODEL_NAME = 'model'  # Directory of Model
    

    # Parse input image name and directory. 
    IM_NAME = directory + image_name
    
    #IM_DIR = args.imagedir

    # If both an image AND a folder are specified, throw an error
    """
    if (IM_NAME and IM_DIR):
        print('Error! Please only use the --image argument or the --imagedir argument, not both. Issue "python TFLite_detection_image.py -h" for help.')
        sys.exit()

    # If neither an image or a folder are specified, default to using 'test1.jpg' for image name
    if (not IM_NAME and not IM_DIR):
        IM_NAME = 'test1.jpg'
    """
    # Import TensorFlow libraries
    # If tflite_runtime is installed, import interpreter from tflite_runtime, else import from regular tensorflow
    # If using Coral Edge TPU, import the load_delegate library
    pkg = importlib.util.find_spec('tflite_runtime')
    if pkg:
        from tflite_runtime.interpreter import Interpreter
        if use_TPU:
            from tflite_runtime.interpreter import load_delegate
    else:
        from tensorflow.lite.python.interpreter import Interpreter
        if use_TPU:
            from tensorflow.lite.python.interpreter import load_delegate

    # If using Edge TPU, assign filename for Edge TPU model
    if use_TPU:
        # If user has specified the name of the .tflite file, use that name, otherwise use default 'edgetpu.tflite'
        if (GRAPH_NAME == 'detect.tflite'):
            GRAPH_NAME = 'edgetpu.tflite'


    # Get path to current working directory
    CWD_PATH = os.getcwd()

    # Define path to images and grab all image filenames
    """
    if IM_DIR:
        PATH_TO_IMAGES = os.path.join(CWD_PATH,IM_DIR)
        images = glob.glob(PATH_TO_IMAGES + '/*')

    elif IM_NAME:
        PATH_TO_IMAGES = os.path.join(CWD_PATH,IM_NAME)
        images = glob.glob(PATH_TO_IMAGES)
    """

    PATH_TO_IMAGES = os.path.join(CWD_PATH,IM_NAME)
    images = glob.glob(PATH_TO_IMAGES)

    # Path to .tflite file, which contains the model that is used for object detection
    PATH_TO_CKPT = os.path.join(CWD_PATH,MODEL_NAME,GRAPH_NAME)

    # Path to label map file
    PATH_TO_LABELS = os.path.join(CWD_PATH,MODEL_NAME,LABELMAP_NAME)

    # Load the label map
    with open(PATH_TO_LABELS, 'r') as f:
        labels = [line.strip() for line in f.readlines()]

    # Have to do a weird fix for label map if using the COCO "starter model" from
    # https://www.tensorflow.org/lite/models/object_detection/overview
    # First label is '???', which has to be removed.
    if labels[0] == '???':
        del(labels[0])

    # Load the Tensorflow Lite model.
    # If using Edge TPU, use special load_delegate argument
    if use_TPU:
        interpreter = Interpreter(model_path=PATH_TO_CKPT,
                                experimental_delegates=[load_delegate('libedgetpu.so.1.0')])
        print(PATH_TO_CKPT)
    else:
        interpreter = Interpreter(model_path=PATH_TO_CKPT)

    interpreter.allocate_tensors()

    # Get model details
    input_details = interpreter.get_input_details()
    output_details = interpreter.get_output_details()
    height = input_details[0]['shape'][1]
    width = input_details[0]['shape'][2]

    floating_model = (input_details[0]['dtype'] == np.float32)

    input_mean = 127.5
    input_std = 127.5

    # Loop over every image and perform detection
    for image_path in images:

        # Load image and resize to expected shape [1xHxWx3]
        image = cv2.imread(image_path)
        image_rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        imH, imW, _ = image.shape 
        image_resized = cv2.resize(image_rgb, (width, height))
        input_data = np.expand_dims(image_resized, axis=0)

        # Normalize pixel values if using a floating model (i.e. if model is non-quantized)
        if floating_model:
            input_data = (np.float32(input_data) - input_mean) / input_std

        # Perform the actual detection by running the model with the image as input
        interpreter.set_tensor(input_details[0]['index'],input_data)
        interpreter.invoke()

        # Retrieve detection results
        boxes = interpreter.get_tensor(output_details[0]['index'])[0] # Bounding box coordinates of detected objects
        classes = interpreter.get_tensor(output_details[1]['index'])[0] # Class index of detected objects
        scores = interpreter.get_tensor(output_details[2]['index'])[0] # Confidence of detected objects
        #num = interpreter.get_tensor(output_details[3]['index'])[0]  # Total number of detected objects (inaccurate and not needed)


        human_detected = False

        # Loop over all detections and draw detection box if confidence is above minimum threshold
        for i in range(len(scores)):
            if ((scores[i] > min_conf_threshold) and (scores[i] <= 1.0)):

                # Get bounding box coordinates and draw box
                # Interpreter can return coordinates that are outside of image dimensions, need to force them to be within image using max() and min()
                ymin = int(max(1,(boxes[i][0] * imH)))
                xmin = int(max(1,(boxes[i][1] * imW)))
                ymax = int(min(imH,(boxes[i][2] * imH)))
                xmax = int(min(imW,(boxes[i][3] * imW)))
                
                image = cv2.rectangle(image, (xmin,ymin), (xmax,ymax), (10, 255, 0), 2)

                # Draw label
                object_name = labels[int(classes[i])] # Look up object name from "labels" array using class index
                label = '%s: %d%%' % (object_name, int(scores[i]*100)) # Example: 'person: 72%'
                labelSize, baseLine = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, 0.7, 2) # Get font size
                label_ymin = max(ymin, labelSize[1] + 10) # Make sure not to draw label too close to top of window
                image = cv2.rectangle(image, (xmin, label_ymin-labelSize[1]-10), (xmin+labelSize[0], label_ymin+baseLine-10), (255, 255, 255), cv2.FILLED) # Draw white box to put label text in
                image = cv2.putText(image, label, (xmin, label_ymin-7), cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0, 0, 0), 2) # Draw label text

                print(labels[int(classes[i])], scores[i])

                if (labels[int(classes[i])] == 'person'):
                    human_detected = True

        # All the results have been drawn on the image, now display the image
        #cv2.imshow('Object detector', image)
        
        cv2.imwrite("img_cv2/" + 'cv2_'  +image_name, image) 

        """
        # Press any key to continue to next image, or press 'q' to quit
        if cv2.waitKey(0) == ord('q'):
            break
        """
    
    # Clean up
    #cv2.destroyAllWindows()

    return human_detected

app = Flask(__name__)

url_dict = {1 : 'http://Ip of Camera No.1:8090', 2 : 'http://Ip of Camera No.2:8090'}

counter = 1

print('Starting')

def RGB565toRGB888(rgb565): # Input as bytearray # Return as bytes 
    if (len(rgb565)%2 != 0):
        print("Size of rgb565 can't be divided by 2 bytes") 
        return
    else:
        counter = 0
        rgb888 = b''

        for i in range(len(rgb565)//2):
            chunk = rgb565[counter:counter+2] # 2 bytes per chunk
            chunk = int.from_bytes(chunk,"little") 
            #print(chunk)
            r = chunk&0b1111100000000000
            r = r >> 11
            r = int(255 / 31 * r + 0.5)
            g = chunk&0b0000011111100000
            g = g >> 5
            g = int(255 / 63 * g + 0.5)
            b = chunk&0b0000000000011111
            b = int(255 / 31 * b + 0.5)
            rgb888 += bytes([r,])
            rgb888 += bytes([g,])
            rgb888 += bytes([b,])
            counter += 2

        return rgb888

@app.route('/detected', methods = ["POST"]) # Receiving POST from sensors

def detected():

    global counter
    human_detected = False
    re = request.data.decode("utf-8") 
    print('Sensor No.' + re + ' has detected something.' )

    # time.sleep(1) # Getting images from cameras was too fast that people couldn't get out from cameras' FOV in time, so cameras may always detect people, although people are leaving.

    for i in url_dict.keys():
        print('Retrieving image from Camera No.' + str(i)) # Retrieving images
        re = requests.get(url_dict[i] + '/frame', stream = True) # Sending GET to cameras
        if (re.status_code == 200):
            print('HTTP Code = 200')
            
            file_name = 'Camera No.' + str(i) + '_' + str(counter) + '.bmp'
            
            # @@@@@@@@@@ Code From https://stackoverflow.com/questions/34611296/how-to-add-a-header-to-a-bmp-image-without-header/34769417#34769417
            # Using Image Library From Pillow

            # In my case, I could've just join raw image data in rgb565 format with BMP header and save it using shutil, 
            # but I couldn't think of that before commenting this.

            with open('img/' + file_name,'wb') as out_file:
                re.raw.decode_content = True # Don't know if this line helps anything
                ba = bytearray(re.content)
                rgb888 = RGB565toRGB888(ba) # Converting raw image data to RGB888
                red, green, blue = Image.frombytes('RGB', (160, 120), rgb888).split()
                image = Image.merge('RGB', (red, green, blue))
                image = image.transpose(Image.FLIP_TOP_BOTTOM)
                image.save('img/' + file_name)
            del re

            print('Starting tf')

            tmp = tf(file_name, 'img/', counter, i)
            
            if (tmp):
                human_detected = True


        else:
            print('HTTP Code = ' + str(re.status_code))
            print('Skipping Camera No.' + str(i))
    
    counter += 1

    if (human_detected == True): # If humans detected, send POST to cameras to open

        for i in url_dict.keys():
            print('Telling Camera No.' + str(i) + ' to open camera' )
            esp = requests.post(url_dict[i] + '/noti', data={'mode':1})
            print('Camera No.' + str(i) + ' HTTP Code is ' + str(esp.status_code))

    elif (human_detected == False): # If none humans detected, send POST to cameras to close

        for i in url_dict.keys():
            print('Telling Camera No.' + str(i) + ' to close camera' )
            esp = requests.post(url_dict[i] + '/noti', data={'mode':0})
            print('Camera No.' + str(i) + ' HTTP code is ' + str(esp.status_code))

    return 'OK'

app.run(host='0.0.0.0', port= 8090) # Listen on port 8090

